import solar_base
import time_date
import math

"""
@version: 0.1
@author: Masatoshi Nishioka <nishioka@urban.eng.osaka-cu.ac.jp>
"""
__version__ = 0.1
__author__ = 'Masatoshi Nishioka <nishioka@urban.eng.osaka-cu.ac.jp>'


__docformat__ = 'epytext jp'

def example_01():
    latitude = 34.68   #Osaka (degree)
    longitude = 135.62
    nday = 1           # Date: 1-Jan
    minute = 0
    ta = 20.0  # Atmospheric Temperature (deg C)
    xa = 0.01  # Atmospheric Humidity (kg/kg(DA))
    sol = SolarBase(latitude, longitude) 
    alpha = 0.0 # ̤̳ (deg)
    beta = 30.0 # ̤ηг (deg)
    print "Date : %02d/%02d" % time_date.jdy_to_date(nday)
    print "%5s %10s %10s %10s" % ("Time", "IDN", "ISKY", "IATM")
    for i in range(1, 25):
        sol.set_time(nday, i, minute)
        sh_s = sol.sh_star(alpha, beta)
        idn, isky, iatm = sol.radiation_h_model(ta, xa)
        print "%02d:%02d %10.3f %10.3f %10.3f" % (i, minute, idn*sh_s, isky, iatm)

    
class SolarBase:
    """۰, ͤδ÷׻
    """
    __PP = 0.65 # 絤ƩΨ
    
    def __init__(self, lat_deg, long_deg, pp=__PP):
        """
        ˥饤

        @param lat_deg: ̰ (deg)
        @param long_deg:  (deg)
        @param pp: 絤ƩΨ (-)
        @return:  None
        """
        self.__rrad = math.pi/180.0 
        self.set_location(lat_deg, long_deg)
        self.__alpha = 0.0
        self.__beta = 0.0
	self.__nday = 1
	self.__hh = 1
	self.__mm = 0
	self.__ss = 0
        self.set_time(1)
        _SolarBase__PP = pp # 絤ƩΨ

    def set_location(self, lat_deg, long_deg):
        """
        ׻о

        @param lat_deg: ̰ (deg)
        @param long_deg:  (deg)
        @return:  None

        """
        self.__lat = lat_deg*self.__rrad
        self.__long = long_deg*self.__rrad

    def set_time(self, nday, hh = 0, mm = 0, ss = 0):
        """
        ׻о

        @param nday: , 11Ȥǯ̻Ϳ. 
                 (㤨С 1231 nday=365)
        @param hh:  
        @param mm:  ʬ
        @param ss:  
        @return:  none
        """
	self.__nday = nday
	self.__hh = hh
	self.__mm = mm
	self.__ss = ss
	
	tt = hh+mm/60.0
        self.__sh, self.__chsa, self.__chca =  solar_base.solar_position_h(nday, tt, self.__lat, self.__long)

    def sh_star(self, alpha=0.0, beta=0.0):
        """
        ̤鸫۹ M{sin(h*)}

        @param alpha: ̤̳,0ȤؤޤȤ (rad)
        @param beta:  ̤ηг (rad)
        @return:      M{sin(h*)}

        """
        self.__alpha = alpha
        self.__beta = beta
        self.__sh_star = solar_base.sh_s(self.__sh, self.__chsa, self.__chca,
                                         alpha*self.__rrad, beta*self.__rrad)
        return self.__sh_star
        
    def radiation_h_model(self, ta=20.0,  xa=0.01): 
        """
        ľã,ŷ,絤ͤΥǥ뼰ˤ׻֤ͤ.
        Ѥǥ뼰,
           - ľã : Bouguerμ
           - ŷ : Watanabeμ
           - 絤 : Yamamoto-Bruntμ


        @param ta:   (deg C)
        @param xa:  м (kg/kg(DA))
        @return: (idn, isky, iatm)
          - idn  : ˡľã (W/m2),
          - isky : ŷ (W/m2),
          - iatm : 絤 (W/m2)

        """
	i0 = solar_base.int_0(self.__nday)
	self.__idn = solar_base.int_dn_Bouguer(i0, self.__PP, self.__sh)
	#self.__isky = solar_base.int_sky_Berlage(i0, self.__PP, self.__sh, self.__idn)
	self.__isky =  solar_base.int_sky_Watanabe(i0, self.__PP, self.__sh)
	self.__atm = solar_base.int_atm_Yamamoto_Brunt(ta, xa);
        return self.__idn, self.__isky, self.__atm
        

if __name__ == '__main__':
    example_01()
    
