from math import sqrt, log, atan2, pi, atan

def test_01():
  td = 26.0 # 岹 (deg C)
  rh = 45.6 # м ( % )
  print "td   : %8.3f (deg C) rh : %8.3f (%%) " % (td, rh)
  xx = rh_to_x(td, rh)
  print "    xx=rh_to_x(td, rh)           :  %8.2e (kg/kg(DA))" %  xx
  ff = x_to_mmHg(xx)
  print "    ff=x_to_mmHg(ss)             :  %8.2e (mmHg) " % ff
  hh = enthalpy(td, xx)
  print "    hh=enthalpy(td, xx)          :  %8.2e (kJ/kg) ****" % hh
  print "    dpf_acss(xx)                 :  %8.2e (deg C)" % dpf_acss(xx)
  print "    twf_acss(hh, xx)             :  %8.2e (deg C)" % twf_acss(hh, xx)
  print "    specific_volume(td, xx)      :  %8.2e (m3/kg(DA))" % specific_volume(td, xx)

  print
  ps = p_sat(td)
  print "    ps=p_sat(td)                 :  %8.2e (mmHg)" % ps
  print "    p_sat_kosik_mmHg(td)         :  %8.2e (mmHg)" % p_sat_kosik_mmHg(td)
  print "    grad_p_sat_kosik(td)         :  %8.2e (kg/(kg(DA) K))" % grad_p_sat_kosik(td)
  print "    x_to_mmHg(p_sat_kosik_x(td)) :  %8.2e (mmHg) " % x_to_mmHg(p_sat_kosik_x(td))
  print "    mmHg_to_Pa(ps)               :  %8.2e (Pa)" % mmHg_to_Pa(ps)
  print "    Pa_to_mmHg(mmHg_to_Pa(ps))   :  %8.2e (mmHg)" % Pa_to_mmHg(mmHg_to_Pa(ps))
  print "    mmHg_to_x(ps)                :  %8.2e (kg/kg(DA))" % mmHg_to_x(ps)
  print "    x_to_mmHg(mmHg_to_x(ps))     :  %8.2e (mmHg)" % x_to_mmHg(mmHg_to_x(ps))


  print
  pa = 760.0 # (mmHg)
  print "    density_air_dry(td, pa)      :  %8.3f (kg/m3)" % density_air_dry(td, pa)
  print "    density_air_humid(td, pa, x_to_mmHg(xx))        : %8.3f (kg/m3)" % density_air_humid(td, pa, x_to_mmHg(xx))
  print "    temp_potential(td, pa)       :  %8.3f (deg C)" % temp_potential(td, pa)

  print
  xx = 1.0
  yy = 1.0
  zz = 0.5
  print " xx=%8.3e yy = %8.3e zz=%8.3e" % (xx,yy,zz)
  print "    form_factor_analytic_ptof_p(xx, yy, zz)   : %8.3f " % form_factor_analytic_ptof_p(xx, yy, zz)
  print "    form_factor_analytic_ptof_n(xx, yy, zz)   : %8.3f " % form_factor_analytic_ptof_n(xx, yy, zz)
  print "    form_factor_analytic_ftof_p(xx, yy, zz)   : %8.3f " % form_factor_analytic_ftof_p(xx, yy, zz)
  print "    form_factor_analytic_ftof_n(xx, yy, zz)   : %8.3f " % form_factor_analytic_ftof_n(xx, yy, zz)
  
def rh_to_x(td, rh):
  """
  м٤м٤ؤѴ

  @param td:  dry bulbe temperature ( deg c ) 
  @param rh:  relative humidity ( % )

  @return:  humiduty ratio ( kg/kg(DA) )

  """
  ps1 = p_sat(td)
  rh1 = rh/100.0
  return  622.0*rh1*ps1/(760.0-rh1*ps1)*0.001


def p_sat_x(td):
  return mmHg_to_x(p_sat(td))
  
def p_sat(td):
  """
  Saturated vapor pressure of moist atmosphere, ˰¿

  Goff-Gratch's equation

  @param td: dry bulbe temperature ( deg C )
  @return:   saturated vapor pressure  ( mmHg )
  """
  if td >= 0.0:
    z = 373.16/(273.16+td)
    y = z-1.0
    x = (-7.90298*y+5.02808*log(z)/log(10.0)
         -1.3816e-7*( pow( 10.0, 11.344*(y/z)) -1.0 )
         +8.13280e-3*( pow( 10.0,-3.49149*y  ) -1.0 ))
  else:
    z = 273.16/(273.16+td)
    y = z-1.0
    x = (-9.09718*y-3.56654*log(z)/log(10.0)
         +0.876793*(y/z)-2.21988)

  return 760.0*pow(10.0,x)

def p_sat_kosik_mmHg(td):
  """
  tdˤ˰¿ [ mmHg], Kosikˤ

  @param  td: 岹  ( deg C ) 
  @return:  ˰¿  ( mmHg )
  """
  if td<0.0: 
      k = 8.439 
  else: 
      k = 7.493

  return 4.58*pow(10.0, k*td/(237.0+td))

def grad_p_sat_kosik_x(td):
  """
  tdˤ롤˰м٤θ ---Kosik ˤ

  @param  td: 岹  ( deg C )
  @return: ˰м٤θ  ( kg/(kg(DA) deg C) )
  """
  if td<0.0:
      k = 8.439
  else:
      k = 7.493

  w1 = k*td/(237.0+td)
  w2 = pow(10.0, w1)
  w3 = w2*log(10.0)*k*237.0/((237.0+td)*(237.0+td))
  return 622.0*4.58*760.0*w3/((760.0-4.58*w2)*(760.0-4.58*w2))*0.001


def p_sat_kosik_x(td):
  """
  tdˤм, Kosikˤ.

  @param td: 岹  ( deg C )
  @return: м ( kg/kg(DA) )
  """
  mmhg = p_sat_kosik_mmHg(td)
  xx = mmHg_to_x(mmhg)
  return xx


def enthalpy(tt, xx):
  """
  󥿥ԡ

  @param tt: 岹 ( deg C )
  @param xx: м ( kg/kg(DA) )
  @return: 󥿥ԡ ( kJ/kg )
  """
  ha = 0.24            # Ǯ kcal/kg degC
  hv = 597.3+0.441*tt   
  return 4.1855*(ha*tt+xx*hv)

def mmHg_to_Pa(p):
  """
  ñѴ  Pa -> mmHg.

  ɸ絤 = 760 mmHg() = 101325 Pa

  @param  p: 絤 ( mmHg )
  @return: 絤 ( Pa )
  """
  return p*133.322

def Pa_to_mmHg(pp):
  """
  ñѴ mmHg -> Pa

  ɸ絤 = 760 mmHg() = 101325 Pa

  @param  pp: 絤 ( Pa )
  @return: 絤 ( mmHg )
  """
  return pp/133.322

def mmHg_to_x(pp):
  """
  м٤Ѵ

  @param pp:   ( mmHg )
  @return:  м ( kg/kg(DA) )
  """
  return (622.0*pp/(760.0-pp))*0.001

def x_to_mmHg(xx):
  """м٤Ѵ

  @param xx:  м ( kg/kg(DA) )
  @return:   ( mmHg )
  """
  xx = xx*1000.0
  return xx/(622.0+xx)*760.0

def dpf_acss(xx):
  """Ϫ

     - ѥ᡼Ŭϰ :м x : 0.001 - 0.035 (kg/kg'),  0.007
     - HASP/ACSS ޤζ
     - HASP/ACSS/8502ץ, ()ѼԶ, 1992.04, pp155-156

  @param xx: м ( kg/kg(DA) )
  @return:   Ϫ ( deg C )
  """
  logxx = log(xx)
  r = (-4.687738e-3*logxx**5 -1.346113e-1*logxx**4
       -1.461462*logxx**3 -6.833415*logxx**2
       +2.348375*logxx +7.710049e1
       +0.8045921*abs(logxx+5.580150))
  return r

def twf_acss(hh, xx):
  """岹

     - ѥ᡼Ŭϰ :   : 0 - 35 (deg C),  0.017
     - HASP/ACSS ޤζ
     - HASP/ACSS/8502ץ, ()ѼԶ, 1992.04, pp155-156

  @param  hh: 󥿥ԡ (kJ/kg)
  @param  xx: м 0.001 - 0.035 ( kg/kg(DA) )
  @return: 岹  ( deg C )
  """
  hh = hh/4.1855 # kJ/kg -> kcal/kg, ʲa1,a2Ѥenthalpy kcal/kg
  a1 = (2.766601e-7*hh**5  -4.048901e-5*hh**4
        +2.586479e-3*hh**3 -9.921376e-2*hh**2
        +2.897049*hh -6.0524)
  a2 = (1.0 + xx*(1.107358e-6*hh**4 -1.433037e-4*hh**3
                  +7.337560e-3*hh**2 -1.948023e-1*hh
                  +2.887669))
  return a1/a2

        
def specific_volume(tt, xx):
  """
  

  @param xx: м (kg/kg(DA))
  @param tt:  (deg C)
  @return:  (m3/kg(DA))
  """
  s = 0.00455*(0.622+xx)*(tt+273.15)
  #pa = 760.0 # mmHg
  #ff = x_to_mmHg(xx)
  #s = 1.0/density_air_humid(tt, pa, ff)
  return s

def density_air_dry(tt, pa):
  """
  ̩().

  ǯɽ1990,p443

  @param tt:  (deg C)
  @param pa:  (mmHg)
  @return:   ̩  (kg/m3)
  """
  r = (1.293e-3/(1.0+0.00367*tt))*pa/760.0*1000.0
  return r

def density_air_humid(tt, pa, pp):
  """
  ̩().

  ǯɽ1990,p443

  @param  tt:  (deg C)
  @param  pa:  (mmHg)
  @param  pp:  (mmHg)
  @return:    ̩  ( kg/m3 )
  """
  r = density_air_dry(tt, pa)*(1.0-0.378*pp/pa)
  return r

def temp_potential(tt, pa):
  """
  (ݥƥ󥷥벹).
  
  :ݤ狼ȼ,Ohmsha,2000

  @param  tt:  (K)
  @param  pa:  (mmHg)
  @return:     (K)
  """
  r = tt*(760.0/pa)**0.286
  return r

def form_factor_analytic_ptof_p(xx, yy, zz): 
  """
  лͳѷηַβϲ(ʿ)

  㤨,,¼:۴Ķ,̽,p70

  @param xx: ͳѷդĹ
  @param yy: ͳѷդĹ
  @param zz: ¬ȻͳѷεΥ
  @return: ַ
  """
  xx1 = xx/zz
  yy1 = yy/zz
  xx2 = sqrt(xx1*xx1+1.0)
  yy2 = sqrt(yy1*yy1+1.0)
  return (((xx1/xx2)*atan2(yy1,xx2)+(yy1/yy2)*atan2(xx1,yy2))/(2.0*pi))


def form_factor_analytic_ptof_n(xx, yy, zz):
  """
  лͳѷηַβϲ(ľ).

  㤨,,¼:۴Ķ,̽,p71

  @param xx: ͳѷդĹ
  @param yy: ͳѷդĹ
  @param zz: ¬ȻͳѷεΥ
  @return:  ַ
  """
  if xx < yy:
    tmp = yy
    yy = xx
    xx = tmp
  xx1 = xx/zz
  yy1 = yy/zz
  xx2 = sqrt(xx1*xx1+1.0)
  yy2 = sqrt(yy1*yy1+1.0)
  return ((atan(xx1)-(1.0/yy2)*atan2(xx1,yy2))/(2.0*pi))

def form_factor_analytic_ftof_p(xx, yy, zz): 
  """
  ĹĹַ(ʿ)

  㤨,,¼:۴Ķ,̽,p233

  @param xx: ͽ̤ޤ(Ĺ)Ĺ.
                - ͽ̤̤ƱˡǤ, xx,yyĹ2դ.
  @param yy: Ʊ
  @param zz: ͽ(Ĺ)(Ĺ)ȤεΥ

  @return: ַ
  """
  x2 = xx*xx;  y2 = yy*yy;  z2 = zz*zz
  a1 = yy*sqrt(x2+z2)*atan2(yy,sqrt(x2+z2))
  a2 = xx*sqrt(y2+z2)*atan2(xx,sqrt(y2+z2))
  a3 = -yy*zz*atan2(yy,zz) - xx*zz*atan2(xx,zz)
  a4 = -z2/2.0*log(z2*(x2+y2+z2)/((x2+z2)*(y2+z2)))
  aa = 2.0/(pi*xx*yy)*(a1+a2+a3+a4)
  return aa

def form_factor_analytic_ftof_n(xx, yy, zz): 
  """
  ĹĹַ(ľ)

  㤨,,¼:۴Ķ,̽,p234

  @param xx: ͽ̤ޤ(Ĺ)Ĺ.
             - ͽ̤ޤ̤, xx,yyĹ2դʤĹ,
               xx, zzĹ2դʤĹǤ.
               xx,ͽ̤̤ˤƶͭ.
  @param yy: Ʊ
  @param zz: Ʊ
  
  @return: ַ
  """
  x2 = xx*xx;  y2 = yy*yy;  z2 = zz*zz
  a1 = yy*zz*atan2(yy,zz) - yy*sqrt(x2+z2)*atan2(yy,sqrt(x2+z2))+xx*yy*atan2(yy,xx)
  a2 =  z2/2.0*log(zz*sqrt(x2+y2+z2)/(sqrt(x2+z2)*sqrt(y2+z2)))
  a3 =  x2/2.0*log(xx*sqrt(x2+y2+z2)/(sqrt(x2+y2)*sqrt(x2+z2)))
  a4 = -y2/2.0*log(yy*sqrt(x2+y2+z2)/(sqrt(x2+y2)*sqrt(y2+z2)))
  aa = 1.0/(pi*xx*yy)*(a1+a2+a3+a4)
  return aa

if __name__ == '__main__':
    test_01()

