from math import sin, cos, pi, sqrt

import math_plus 
import ee_base

# Constant
_C_SE = 1370.0      # Solar Constant (W/m^2)
_C_SB = 5.67E-8     # Stefan-Boltzmann constant  (W/(m^2 K^4)])

def solar_position_h(nday, tt, lat, lon):
    """
    ʿ̤鸫۰֤η׻.

    Solar Position on holizontal surface.

    @param nday: serial number of day in a year, 
              ׻ǯֹ֤̤ (1365)
    @param tt:    
    @param lat: latitude    ( rad )
    @param lon: longitude   ( rad )

    @return:  (sh, chsa, chca)
       - sh:   M{sin(h)}         ( rad )
            - note) HASPλɽϡ124Ǥ롥 
       - chsa:  M{cos(h)sin(A)}  ( rad )
       - chca:  M{cos(h)cos(A)}  ( rad )
    """

    om = 2.0*pi*nday/366.0           
    dt = (0.006322 -0.405748*cos(    om+0.153231) 
          -0.005880*cos(2.0*om+ 0.207099) 
          -0.003233*cos(3.0*om+0.620129))   # ְ (rad) 
    et = (-0.000279 +0.122772*cos(    om+1.498311) 
          -0.165458*cos(2.0*om-1.261546) 
          -0.005354*cos(3.0*om-1.157100))   # ѻ  (h)

    s_dt=sin(dt);    c_dt=cos(dt)
    s_lat=sin(lat);  c_lat=cos(lat)

    tt2 =pi/12.0*((tt+et-12.0)+(lon/pi*180-135.0)/15.0) 

    sh   = s_lat*s_dt +c_lat*c_dt*cos(tt2)
    chsa = c_dt*sin(tt2)
    chca =-s_dt*c_lat +c_dt*s_lat*cos(tt2)

    if sh<0.0:
        sh = 0.0
        
    return sh, chsa, chca
    
def solar_position(nday, tt, lat, lon):
    sh, chsa, chca = solar_position_h(nday, tt, lat, lon)
    alpha = 0.0
    beta = 0.0
    return sh_s(sh, chca, chsa, alpha, beta)

def sh_s(sh, chsa, chca, alpha, beta):
    """
    Ǥդη̤鸫۹.

    @param sh: M{sin(h)}            
       - note) HASPλɽϡ124Ǥ롥 
    @param chsa:  M{cos(h)sin(A)} 
    @param chca:  M{cos(h)cos(A)} 
    @param alpha: ̳
    @param beta: г
    @return: M{sin(h*)}
    """
    s = sh*cos(beta)+chca*cos(alpha)*sin(beta)+chsa*sin(alpha)*sin(beta)
    if s<0.0:
        s = 0.0
    return s

def int_sky_Watanabe(i0, p, sh):
    """
    դμˤʿŷ. 

    @param i0:  
    @param p:
    @param sh:
    @return:
         ñ̤ i0 ˽
    """
    if sh > 0.0:
        mm = 1.0/sh
        q0 = 1.0-pow(p,mm)
        q1 = (0.8672+0.7505*sh)*pow(p,(0.421*mm))*pow(q0,2.277)
        r = i0*sh*q1/(1.0+q1)
    else: 
        r =0.0
    return r

def int_0(nday):
    """
    絤.
    Intensity of solar radiation.

    @param nday: serial number of the day
    @return: ñ̤,˽
    """
    return _C_SE*(1.0+0.033*cos(2.0*pi*nday/365.0))

def int_dn_Bouguer(i0, pp, sh):
    """
    Bouguerˤˡľã.
    Intensity of direct solar radiation,  by Bouguer's equation.

    @param i0: intensity of solar radiation in the outside of atmosphere
    @param pp: 絤ƩΨ
    @param  sh: M{sin(h)}, M{h}: heighy of sol
    @return:
         ñ̤ i0 ˽
    """
    if sh>0.0: 
        r = i0*pow(pp,(1.0/sh))
    else:
        r = 0.0

    return r

def int_sky_Berlage(i0, p, sh, idn):
    """
    Berlageˤʿŷ.
    Intensity of sky solar radiation,  by Berlage's equation.

    @param i0: intensity of solar radiation in the outside of atmosphere
    @param p: 絤ƩΨ
    @param sh: M{sin(h)}, M{h}: height of sol
    @param idn: intensity of direct solar radiation

    @return:
          ñ̤ i0,idn ˽
    """
    if sh > 0.0:
        sh0 = sh*(i0-idn)
        r = sh0*0.5/(1.0-1.4*log(p))
    else:
        r = 0.0
    return r

def int_sky_Nagata(i0, p, sh, idn): 
    """
    ļˤʿŷ. 

    @param  i0: 
    @param  p: 絤ƩΨ
    @param  sh: M{sin(h)}
    @param idn: ľã

    @return:
           ñ̤ i0,idn ˽
    """
    
    if sh > 0.0:
        sh0 = sh*(i0-idn)
        r = sh0*(0.5+(0.4-0.3*p)*sh)*(0.66-0.32*sh)
    else:
        r = 0.0
    return r


def int_noc_HASP(t0, xx, ct):
    """
    .
    Nocturnal radiation,   < cf. HASP 8001 >

    @param t0: temperature    ( deg C )
    @param xx: humidity ratio ( kg/kg(DA) )
    @param ct:  cloudness      ( - )

    @return:
         ñ : ( W m^(-2) K^(-1) )
    """

    #ñ̤ Stefan-Boltzmann˽
    sb = _C_SB
    #sb = 4.88E-8
    tt = 273.15+t0
    xx = xx*1000.0
    return (sb*tt**4 *(1.0-0.062*ct)*
            (0.49-2.1*sqrt(xx/(xx+622.0))))

def int_noc_Kentiku(t0, xx, ct): 
    """
    . Nocturnal radiation, 

    ¾:۴Ķ,pp214-215

    @param t0: temperature    ( deg C )
    @param x: humidity ratio ( kg/kg(DA) )
    @param ct: cloudness      ( - )

    @return: ñ̤Stefan-Boltzmann constantˤ 
    """
    kk = 0.3   # Ƿޤ뷸,ر:0.8,ر:0.3,ر:0.15
    sb = _C_SB
    #sb = 4.88E-8
    xx = xx*1000.0
    jn = (sb*pow((273.15+t0) ,4.0)*
          (0.474-0.076*sqrt(eeng_base.x_to_mmHg(xx))))
    jc = jn*(1-(1-kk)*ct/10)
    return jc


def int_atm_Yamamoto_Brunt(ta, xx):
    """
    Yamamoto-Bruntˤ絤

    cf. 絤ʳعֺ1,p87.
    θ̤θƤʤȤʤ뤫Τʤ
    ()

    @param ta: ؤʿѲ (deg C)
    @param xx: ؤʿѼ ( kg/kg (DA))
       mbѴ M{x/((622.0+x)*0.750062)*760.0}, 0.750062mmHg = 1mb
    @return:
          (kcal/m^2h )
    """
    #sb = 4.88E-8
    sb = _C_SB
    tt = (ta+273.15)
    xx = xx*1000.0
    return  (sb*tt**4 *(0.51+0.066*sqrt(1013.0*xx/(622.0+xx))))

def int_atm_Kentiku(ta, xx):
    """
    絤.

    ¾:۴Ķ,pp214-215

    @param  ta: 絤 (deg C)
    @param  x:  絤 ( kg/kg (DA))
    @return: ñ:Stefan-Boltzmann constantˤ 
    """
    sb = _C_SB
    #sb = 4.88E-8
    tt = ta+273.15
    xx = xx*1000.0
    return  (sb*tt**4 *(0.526+0.076*sqrt(eeng_base.x_to_mmHg(x))))



